// cone.h is a part of the TEMPLATE Higgs tagger.

// Header file for the Event and Cone classes.
// Event: simple container for the particles in a jet or a template.
// Cone: template analysis of an event.		

#ifndef Cone_H
#define Cone_H

#include <cmath>
#include <string>
#include <vector>
#include "basics.h"
#include "event.h"


using namespace std;



/// Simple helper class to Cone for a particle o pseudo-particle. 

class SingleParticle {

public:

  // Constructors.
  SingleParticle( Vec4 pIn = 0., double pTIn = 0., double yIn = 0., 
    double phiIn = 0.) : p(pIn), pT(pTIn), y(yIn), phi(phiIn), 
    mult(1), isUsed(false) { }
  SingleParticle(const SingleParticle& ssj) : p(ssj.p), pT(ssj.pT),
    y(ssj.y), phi(ssj.phi), mult(ssj.mult), isUsed(ssj.isUsed) { }
  SingleParticle& operator=(const SingleParticle& ssj) { if (this != &ssj)
    { p = ssj.p; pT = ssj.pT; y = ssj.y; phi = ssj.phi; 
    mult = ssj.mult; isUsed = ssj.isUsed;} return *this; }

  // Properties of particle.
  Vec4   p;  
  double pT, y, phi;
  int    mult; 
  bool   isUsed;
  
  double deltaR2(const SingleParticle & other) const ;
  
  void bst (const Vec4 & pBoost);

} ;



/// Main class to perform data analysis. 

class Cone {

public: 

  // Constructor.
  /// Construct a grid of CAL cells with standard values nEta = 50, nPhi = 32.
  // Constructor.
  Cone(double coneRadiusIn = 1.2, double subConeRadiusIn = 0.2, bool ModeIn = 0, double pTjetMinIn = 0., 
    double etaMaxIn = 25. ) : coneRadius (coneRadiusIn), subConeRadius (subConeRadiusIn), Mode (ModeIn),
  
    pTjetMin(pTjetMinIn), etaMax(etaMaxIn) {sigma = subConeRadius/sqrt(2);}
    
  
  // Analyze event.
  // Return pair <peak template, max overlap>
  pair<vector<SingleParticle>, double> analyze(const Event& event, const vector<SingleParticle> & comparison_template = vector<SingleParticle>(), double R0 = 0.
);
   //Add particles pT to a grid.
   const vector<SingleParticle> addEnergyDeposition(const Event& event) const ;  
   //Add particle pT to a single cell. 
   const SingleParticle  addParticleEnergyDeposition(const Vec4& particle) const ;
  
 // Load templates.
  int loadTemplates(ifstream& is);
    
   const vector<SingleParticle>  getTemplate(int i) {return templates[i];}
   
  
  double GetMassInsideCone (const vector<SingleParticle> & probe_template);
  
  double GetOverlap (const vector<SingleParticle> & probe_template);
  double GetOverlapGaussian (const vector<SingleParticle> & probe_template);
    double GetOverlapGaussian2 (const vector<SingleParticle> & probe_template);

  double overlapDistance (const vector<SingleParticle> & jet1, const vector<SingleParticle> & jet2, double R0);
   
   const vector<double> get_all_overlaps() {return all_overlaps;} 

  bool CheckTemplate (const vector<SingleParticle> & probe_template);

 // Provide a listing of the info.
  void list_templates(ostream& os = cout, int Nmax =20000);

void reset();

 
  // Provide a listing of the info.
  void list(ostream& os = cout) const;
  

private: 



 


  // Intermediate clustering objects and final jet objects.
  vector<SingleParticle> my_template;
  vector<vector<SingleParticle> > templates;
  vector<SingleParticle> jets;

vector<double>  all_overlaps;


  // Other intermediate variables.
  
 double   coneRadius, subConeRadius;
 
  bool Mode; 
  
  double sigma;
  
    double  pTjetMin, etaMax ; 


 int  nTemplateParticle;
 
  
  
};  



#endif // end Event_H


