#include <fstream>
#include <string>
#include <algorithm>
#include <cmath>
#include <iostream>
#include <memory.h>
#include "basics.h"
#include "event.h"
#include "cone.h"

#include <sstream> 
#include <utility>
#include <iterator>

// Small number to avoid division by zero.
const double Event::TINY = 1e-20;

//--------------------------------------------------------------------------

// pT ascending (low-high)
bool sortVecByPt(const Vec4 &d1, const Vec4 &d2){
	return d1.pT() < d2.pT();
}
 
/// Compute planar flow jet shape.
double Event::planarFlow() const{
	
  double Ixy = 0.,Ixx = 0.,Iyy = 0.;
  for ( int i = 0; i < size(); ++i) {
  //Define a orthogonal basis.
    Vec4 v1 = cross3(axis(), Vec4(0,0,1,0)); //Should check axis != z?
    Vec4 v2 = cross3(axis(), v1);
    v1 *= 1/sqrt(dot3(v1, v1));
    v2 *= 1/sqrt(dot3(v2, v2));
  //Get x and y components in that basis.
    double px=dot3(particles[i],v1),py=dot3(particles[i],v2),ee=particles[i].e();
  // Calculate matrix to be used in the computation of planar flow
    Ixx += px * px/ee;
    Iyy += py * py/ee;
    Ixy += px * py/ee;				
  }
//Return planar flow.
return(4 * (Ixx * Iyy -Ixy * Ixy)/((Ixx + Iyy )*(Ixx + Iyy )) );
}

/// Compute planar flow jet shape. Boost invariant way. 
double Event::planarFlow2(double r) const{
  double Ixy = 0.,Ixx = 0.,Iyy = 0.;
  double etaJet = axis().eta();
  double phiJet = axis().phi();

  for ( int i = 0; i < size(); ++i) {
    //Get x and y components in that basis.
    double  etaNow = particles[i].eta() - etaJet , phiNow = particles[i].phi() - phiJet,
            ee = particles[i].pT();
    if (abs(phiNow) > M_PI) phiNow = 2 * M_PI - abs(phiNow);
    // Calculate matrix to be used in the computation of planar flow
    
    Ixx += ee * (etaNow  * etaNow + r * r/4) ;  
    Iyy += ee * (phiNow * phiNow + r * r/4) ;
    Ixy += ee * etaNow * phiNow; 
    }         

    //Return planar flow.
    return(4 * (Ixx * Iyy -Ixy * Ixy)/((Ixx + Iyy )*(Ixx + Iyy )) );
}


/// Compute angularity jet shape.

double Event::angularity(int a, double R) const {

//Definition 0807.0234v1

double sum = 0.;

for (int i = 0; i < size(); ++i){
	//double Theta = theta(axis(), particles[i]);

//Changed to make it boost invariant. 
	
	double Theta = max (sqrt(particles[i].plain_distance(axis())), TINY);

	//double cosTheta = cos(M_PI * Theta/(2 * R));
		double cosTheta = cos(Theta);

	double sinTheta = sqrt(1 - cosTheta * cosTheta);

	sum+= particles[i].pT() * pow(sinTheta, a) * pow(1-cosTheta, 1-a);


}

return (1/axis().mCalc() * sum);

}


double Event::thetaS() const {
	

	return (theta(axis(), particles[0]));

}

double Event::thetaBar () const{
	
	double sumTheta = 0.;
	for (int i = 0; i < int(size());++i) 
		sumTheta += sin (theta(axis(), particles[i]));
		
	
	return (sumTheta);
	
	}


double Event::dipolarity(Vec4 j1, Vec4 j2) const{

double eta1=j1.eta(), eta2=j2.eta(),phi1=j1.phi(),phi2=j2.phi();
double aux=0., x2=eta2-eta1, y2=phi2-phi1,dR12=sqrt(j1.plain_distance(j2));
for (int i = 0; i < int(size());++i)
{
	double etaNow=particles[i].eta(), phiNow=particles[i].phi(),pTnow=particles[i].pT();
	double x1=etaNow-eta1,y1=phiNow-phi1;
    double Rnow=(1/dR12)*abs(x1*y2-x2*y1);
	aux += pTnow*Rnow*Rnow;
}

return (1/(axis().pT()*dR12*dR12)*aux);
}


double Event::dipolarity2(Vec4 j1, Vec4 j2) const{

double eta1=j1.eta(), eta2=j2.eta(),phi1=j1.phi(),phi2=j2.phi();
double aux=0., x2=eta2-eta1, y2=phi2-phi1,dR12=sqrt(j1.plain_distance(j2));
for (int i = 0; i < int(size());++i)
{
	double etaNow=particles[i].eta(), phiNow=particles[i].phi();
	double x1=etaNow-eta1,y1=phiNow-phi1;
    double Rnow=(1/dR12)*abs(x1*y2-x2*y1);
	aux += Rnow*Rnow;
}

return (1/(dR12*dR12)*aux);
}



/// Read Event from a file.

bool Event::readEvent(ifstream& is) {

  string line, tag;
     double pup1, pup2, pup3, pup4, m,pt;

clear();
	

   if (!getline(is, line)) return false;

	//Read information about jet
  istringstream getpro(line);
  int nupSave, nPileup;
  getpro >> nupSave >> nPileup >> pt >> m;

  //Read 4-Vec  jet
   if (!getline(is, line)) return false;
    istringstream getall(line);
    getall  >> pup1 >> pup2 >> pup3 >> pup4 >> m;
 
  /*Read b-jet 4-Vec
    if (!getline(is, line)) return false;
    istringstream getall2(line);
    getall2  >> pup1 >> pup2 >> pup3 >> pup4 >> m;
		bJets.push_back( Vec4( pup1, pup2, pup3, pup4));
    
				//Read  another b-jet 4-Vec
	if (!getline(is, line)) return false;
    istringstream getall3(line);
    getall3  >> pup1 >> pup2 >> pup3 >> pup4 >> m;
 bJets.push_back(Vec4( pup1, pup2, pup3, pup4));

 */
  
  for (int ip = 0; ip < nupSave; ++ip) { 
    if (!getline(is, line)) return false;
    istringstream getall(line);
    getall  >> pup1 >> pup2 >> pup3 >> pup4;

    append( Vec4( pup1, pup2, pup3, pup4) );
  }
  return true;
}

bool Event::readEvent(fastjet::PseudoJet & myJet) {

//clear event record
clear();
vector<fastjet::PseudoJet> my_particles = myJet.constituents();
for (unsigned int ip = 0; ip < my_particles.size(); ++ip) 
    append( Vec4( my_particles[ip].px(), my_particles[ip].py(), my_particles[ip].pz(), my_particles[ip].e()) );
  
// Reading worked.
return true;

}


/*
bool Event::readEvent(vector<SingleParticle> & my_particles) {

clear();
for (unsigned int ip = 0; ip < my_particles.size(); ++ip) 
    append( my_particles[ip].p );
  
// Reading worked.
return true;
}

*/
